{*******************************************************}
{                                                       }
{             Delphi REST Client Framework              }
{                                                       }
{ Copyright(c) 2013-2019 Embarcadero Technologies, Inc. }
{              All rights reserved                      }
{                                                       }
{*******************************************************}
unit frmREST.Authenticator.OAuth.WebForm;

interface

uses
  System.SysUtils, System.Variants, System.Classes, Vcl.Controls, Vcl.StdCtrls, Vcl.ExtCtrls, ActiveX, ShlObj, UrlMon,
  Vcl.Graphics, Vcl.Forms, Vcl.Dialogs, SHDocVw, Vcl.OleCtrls, oleserver;

//const
//  DISPID_AMBIENT_USERAGENT = -5513;
//
//type
//  TWebBrowser = class (SHDocVw.TWebbrowser, IDispatch)
//  private
//    FUserAgent: string;
//    procedure SetUserAgent (const Value: string);
//    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HRESULT; stdcall;
//  public
//    property UserAgent: string read FUserAgent write SetUserAgent;
//    constructor Create(AOwner: TComponent); override;
//  end;
type
  TOAuth2WebFormRedirectEvent = procedure(const AURL: string; var DoCloseWebView : boolean) of object;
  TOAuth2WebFormTitleChangedEvent = procedure(const ATitle: string; var DoCloseWebView : boolean) of object;

  Tfrm_MyOAuthWebForm = class(TForm)
    Bevel1: TBevel;
    btn_Close: TButton;
    Browser: TWebBrowser;
    Label1: TLabel;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure BrowserTitleChange(ASender: TObject; const Text: WideString);
    procedure FormCreate(Sender: TObject);
    procedure BrowserNavigateComplete2(ASender: TObject; const pDisp: IDispatch; const URL: OleVariant);
    procedure BrowserBeforeNavigate2(ASender: TObject; const pDisp: IDispatch; const URL, Flags, TargetFrameName, PostData, Headers: OleVariant; var Cancel: WordBool);
  private
    { Private declarations }
    FOnBeforeRedirect: TOAuth2WebFormRedirectEvent;
    FOnAfterRedirect: TOAuth2WebFormRedirectEvent;
    FOnBrowserTitleChanged : TOAuth2WebFormTitleChangedEvent;

    FLastTitle: string;
    FLastURL: string;
  public
    { Public declarations }
    procedure ShowWithURL(const AURL: string);

    property LastTitle: string read FLastTitle;
    property LastURL: string read FLastURL;

    property OnAfterRedirect: TOAuth2WebFormRedirectEvent read FOnAfterRedirect write FOnAfterRedirect;
    property OnBeforeRedirect: TOAuth2WebFormRedirectEvent read FOnBeforeRedirect write FOnBeforeRedirect;
    property OnTitleChanged : TOAuth2WebFormTitleChangedEvent read FOnBrowserTitleChanged write FOnBrowserTitleChanged;
  end;

var
  frm_MyOAuthWebForm: Tfrm_MyOAuthWebForm;

implementation

{$R *.dfm}

procedure Tfrm_MyOAuthWebForm.BrowserBeforeNavigate2(ASender: TObject;
  const pDisp: IDispatch; const URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
var
  LDoCloseForm : boolean;
begin
  if Assigned(FOnBeforeRedirect) then
  begin
    LDoCloseForm:= FALSE;

    FOnBeforeRedirect(URL, LDoCloseForm);

    if LDoCloseForm then
    begin
      Cancel:= TRUE;
      self.Close;
    end;
  end;
end;

procedure Tfrm_MyOAuthWebForm.BrowserNavigateComplete2(ASender: TObject; const pDisp: IDispatch; const URL: OleVariant);
var
  LDoCloseForm : boolean;
begin
  FLastURL := System.Variants.VarToStrDef(URL, '');

  if Assigned(FOnAfterRedirect) then
  begin
    LDoCloseForm:= FALSE;

    FOnAfterRedirect(FLastURL, LDoCloseForm);

    if LDoCloseForm then
      self.Close;
  end;
end;

procedure Tfrm_MyOAuthWebForm.BrowserTitleChange(ASender: TObject; const Text: WideString);
var LCloseForm : boolean;
begin
  if (Text <> FLastTitle) then
  begin
    FLastTitle := Text;

    if Assigned(FOnBrowserTitleChanged) then
    begin
      LCloseForm:= FALSE;
      FOnBrowserTitleChanged( FLastTitle, LCloseForm );

      if LCloseForm then
        self.Close;
    end;
  end;
end;

procedure Tfrm_MyOAuthWebForm.FormCreate(Sender: TObject);
begin
  FOnAfterRedirect := NIL;
  FOnBeforeRedirect:= NIL;
  FOnBrowserTitleChanged:= NIL;

  FLastTitle := '';
  FLastURL := '';
end;

procedure Tfrm_MyOAuthWebForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #27) then
  begin
    Close;
  end;
end;

procedure Tfrm_MyOAuthWebForm.ShowWithURL(const AURL: string);
//Var
//  PostData : OleVariant;
begin
//  Browser.UserAgent := 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36';
//  PostData:=EmptyParam;
//  Browser.DefaultInterface.Navigate(AURL, EmptyParam, EmptyParam, PostData, 'User-Agent:'+Browser.UserAgent+#13#10);

  Browser.Navigate(AURL);

  Self.ShowModal;
end;

//{ TWebBrowser }
//
//constructor TWebBrowser.Create(AOwner: TComponent);
//begin
//  inherited Create(AOwner);
//  FUserAgent:='';
//end;
//
//function TWebBrowser.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HRESULT;
//begin
//  //check if the DISPID_AMBIENT_USERAGENT flag is being processed and if the User Agent to set is not empty
//  if (FUserAgent <> '') and (Flags and DISPATCH_PROPERTYGET <> 0) and Assigned(VarResult) and (DispId=DISPID_AMBIENT_USERAGENT) then
//  begin
//    //set the user agent
//    POleVariant(VarResult)^:= FUserAgent+#13#10;
//    Result := S_OK; //return S_OK
//  end
//  else
//  Result := inherited Invoke(DispID, IID, LocaleID, Flags, Params, VarResult, ExcepInfo, ArgErr); //call the default Invoke method
//end;
//
//procedure TWebBrowser.SetUserAgent(const Value: string);
//var
//  Control: IOleControl;
//begin
//  FUserAgent := Value;
//  //the current interface supports IOleControl?
//  if DefaultInterface.QueryInterface(IOleControl, Control) = 0 then
//    Control.OnAmbientPropertyChange(DISPID_AMBIENT_USERAGENT); //call the OnAmbientPropertyChange event
//end;

end.
